@@ -7,7 +7,7 @@
  */
 
 /*
- * Copyright (C) 2012, 2013, 2015 the Free Software Foundation, Inc.
+ * Copyright (C) 2012, 2013, 2015, 2018 the Free Software Foundation, Inc.
  *
  * This file is part of GAWK, the GNU implementation of the
  * AWK Programming Language.
@@ -46,7 +46,7 @@
 #define N_(msgid) msgid
 
 static const gawk_api_t *api;	/* for convenience macros to work */
-static awk_ext_id_t *ext_id;
+static awk_ext_id_t ext_id;
 static const char *ext_version = "revoutput extension: version 1.1";
 
 static awk_bool_t init_revoutput(void);
