@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 1986, 1988, 1989, 1991-2001, 2003-2015, 2017, 2018, 2019,
+ * Copyright (C) 1986, 1988, 1989, 1991-2001, 2003-2015, 2017-2019, 2021-2025,
  * the Free Software Foundation, Inc.
  *
  * This file is part of GAWK, the GNU implementation of the
@@ -25,10 +25,7 @@
  */
 
 #include "awk.h"
-#include "math.h"
-#include "floatmagic.h"	/* definition of isnan */
 
-static int is_ieee_magic_val(const char *val);
 static NODE *r_make_number(double x);
 static AWKNUM get_ieee_magic_val(char *val);
 extern NODE **fmt_list;          /* declared in eval.c */
@@ -62,7 +59,13 @@ r_force_number(NODE *n)
 	char *cpend;
 	char save;
 	char *ptr;
-	extern double strtod();
+
+	if (n->type == Node_elem_new) {
+		elem_new_reset(n);
+		n->type = Node_val;
+
+		return n;
+	}
 
 	if ((n->flags & NUMCUR) != 0)
 		return n;
@@ -101,9 +104,12 @@ r_force_number(NODE *n)
 	if (! do_posix) {
 		if (is_alpha((unsigned char) *cp))
 			goto badnum;
-		else if (cpend == cp+4 && is_ieee_magic_val(cp)) {
-			n->numbr = get_ieee_magic_val(cp);
-			goto goodnum;
+		else if (is_ieee_magic_val(cp)) {
+			if (cpend == cp + 4) {
+				n->numbr = get_ieee_magic_val(cp);
+				goto goodnum;
+			} else
+				goto badnum;
 		}
 		/* else
 			fall through */
@@ -165,6 +171,9 @@ badnum:
 	return n;
 
 goodnum:
+	if (isnan(n->numbr) && *cp == '-' && signbit(n->numbr) == 0)
+		n->numbr = -(n->numbr);
+
 	if ((n->flags & USER_INPUT) != 0) {
 		/* leave USER_INPUT enabled to indicate that this is a strnum */
 		n->flags &= ~STRING;
@@ -238,16 +247,16 @@ r_format_val(const char *format, int index, NODE *s)
 		NODE *dummy[2], *r;
 		unsigned int oflags;
 
-		/* create dummy node for a sole use of format_tree */
+		/* create dummy node for a sole use of format_args */
 		dummy[1] = s;
 		oflags = s->flags;
 
 		if (val == s->numbr) {
 			/* integral value, but outside range of %ld, use %.0f */
-			r = format_tree("%.0f", 4, dummy, 2);
+			r = format_args("%.0f", 4, dummy, 2);
 			s->stfmt = STFMT_UNUSED;
 		} else {
-			r = format_tree(format, fmt_list[index]->stlen, dummy, 2);
+			r = format_args(format, fmt_list[index]->stlen, dummy, 2);
 			assert(r != NULL);
 			s->stfmt = index;
 		}
@@ -286,7 +295,7 @@ r_format_val(const char *format, int index, NODE *s)
 	}
 	if ((s->flags & (MALLOC|STRCUR)) == (MALLOC|STRCUR))
 		efree(s->stptr);
-	emalloc(s->stptr, char *, s->stlen + 1, "format_val");
+	emalloc(s->stptr, char *, s->stlen + 1);
 	memcpy(s->stptr, sp, s->stlen + 1);
 no_malloc:
 	s->flags |= STRCUR;
@@ -304,27 +313,23 @@ r_dupnode(NODE *n)
 	assert(n->type == Node_val);
 
 #ifdef GAWKDEBUG
+	/* Do the same as in awk.h:dupnode().  */
 	if ((n->flags & MALLOC) != 0) {
 		n->valref++;
 		return n;
 	}
 #endif
+	getnode(r);
+	*r = *n;
 
 #ifdef HAVE_MPFR
 	if ((n->flags & MPZN) != 0) {
-		r = mpg_integer();
+		mpz_init(r->mpg_i);
 		mpz_set(r->mpg_i, n->mpg_i);
-		r->flags = n->flags;
 	} else if ((n->flags & MPFN) != 0) {
-		r = mpg_float();
+		mpfr_init(r->mpg_numbr);
 		int tval = mpfr_set(r->mpg_numbr, n->mpg_numbr, ROUND_MODE);
 		IEEE_FMT(r->mpg_numbr, tval);
-		r->flags = n->flags;
-	} else {
-#endif
-		getnode(r);
-		*r = *n;
-#ifdef HAVE_MPFR
 	}
 #endif
 
@@ -339,12 +344,13 @@ r_dupnode(NODE *n)
 	r->wstlen = 0;
 
 	if ((n->flags & STRCUR) != 0) {
-		emalloc(r->stptr, char *, n->stlen + 1, "r_dupnode");
+		emalloc(r->stptr, char *, n->stlen + 1);
 		memcpy(r->stptr, n->stptr, n->stlen);
 		r->stptr[n->stlen] = '\0';
+		r->stlen = n->stlen;
 		if ((n->flags & WSTRCUR) != 0) {
 			r->wstlen = n->wstlen;
-			emalloc(r->wstptr, wchar_t *, sizeof(wchar_t) * (n->wstlen + 1), "r_dupnode");
+			emalloc(r->wstptr, wchar_t *, sizeof(wchar_t) * (n->wstlen + 1));
 			memcpy(r->wstptr, n->wstptr, n->wstlen * sizeof(wchar_t));
 			r->wstptr[n->wstlen] = L'\0';
 			r->flags |= WSTRCUR;
@@ -370,7 +376,7 @@ int
 cmp_awknums(const NODE *t1, const NODE *t2)
 {
 	/*
-	 * This routine is also used to sort numeric array indices or values.
+	 * This routine is used to sort numeric array indices or values.
 	 * For the purposes of sorting, NaN is considered greater than
 	 * any other value, and all NaN values are considered equivalent and equal.
 	 * This isn't in compliance with IEEE standard, but compliance w.r.t. NaN
@@ -390,7 +396,6 @@ cmp_awknums(const NODE *t1, const NODE *t2)
 	return 1;
 }
 
-
 /* make_str_node --- make a string node */
 
 NODE *
@@ -398,6 +403,7 @@ make_str_node(const char *s, size_t len, int flags)
 {
 	NODE *r;
 	getnode(r);
+	memset(r, '\0', sizeof(NODE));
 	r->type = Node_val;
 	r->numbr = 0;
 	r->flags = (MALLOC|STRING|STRCUR);
@@ -412,7 +418,7 @@ make_str_node(const char *s, size_t len, int flags)
 	if ((flags & ALREADY_MALLOCED) != 0)
 		r->stptr = (char *) s;
 	else {
-		emalloc(r->stptr, char *, len + 1, "make_str_node");
+		emalloc(r->stptr, char *, len + 1);
 		memcpy(r->stptr, s, len);
 	}
 	r->stptr[len] = '\0';
@@ -434,10 +440,15 @@ make_str_node(const char *s, size_t len, int flags)
 			 * character happens to be a backslash.
 			 */
 			if (gawk_mb_cur_max > 1) {
-				int mblen = mbrlen(pf, end-pf, &cur_state);
+				size_t mblen = mbrlen(pf, end-pf, &cur_state);
 
-				if (mblen > 1) {
-					int i;
+				/*
+				 * Incomplete (-2), invalid (-1), and
+				 * null (0) characters are excluded here.
+				 * They are read as a sequence of bytes.
+				 */
+				if (mblen > 1 && mblen < (size_t) -2) {
+					size_t i;
 
 					for (i = 0; i < mblen; i++)
 						*ptm++ = *pf++;
@@ -447,20 +458,39 @@ make_str_node(const char *s, size_t len, int flags)
 
 			c = *pf++;
 			if (c == '\\') {
-				c = parse_escape(&pf);
-				if (c < 0) {
+				const char *result;
+				size_t nbytes;
+				enum escape_results ret;
+
+				ret = parse_escape(& pf, & result, & nbytes);
+				switch (ret) {
+				case ESCAPE_OK:
+					assert(nbytes > 0);
+					while (nbytes--)
+						*ptm++ = *result++;
+					break;
+				case ESCAPE_CONV_ERR:
+					*ptm++ = '?';
+					break;
+				case ESCAPE_TERM_BACKSLASH:
+					if (do_lint)
+						lintwarn(_("backslash at end of string"));
+					*ptm++ = '\\';
+					break;
+				case ESCAPE_LINE_CONTINUATION:
 					if (do_lint)
 						lintwarn(_("backslash string continuation is not portable"));
-					if ((flags & ELIDE_BACK_NL) != 0)
-						continue;
-					c = '\\';
+					continue;
+				default:
+					cant_happen("received bad result %d from parse_escape(), nbytes = %zu",
+							(int) ret, nbytes);
+					break;
 				}
-				*ptm++ = c;
 			} else
 				*ptm++ = c;
 		}
 		len = ptm - r->stptr;
-		erealloc(r->stptr, char *, len + 1, "make_str_node");
+		erealloc(r->stptr, char *, len + 1);
 		r->stptr[len] = '\0';
 	}
 	r->stlen = len;
@@ -482,8 +512,13 @@ make_typed_regex(const char *re, size_t len)
 
 	n2 = make_string(re, len);
 	n2->typed_re = n;
+#if HAVE_MPFR
+	if (do_mpfr)
+		mpg_zero(n2);
+	else
+#endif
 	n2->numbr = 0;
-	n2->flags |= NUMCUR|STRCUR|REGEX; 
+	n2->flags |= NUMCUR|STRCUR|REGEX;
 	n2->flags &= ~(STRING|NUMBER);
 
 	return n2;
@@ -496,23 +531,31 @@ void
 r_unref(NODE *tmp)
 {
 #ifdef GAWKDEBUG
-	if (tmp == NULL)
+	/* Do the same as in awk.h:unref().  */
+	assert(tmp == NULL || tmp->valref > 0);
+	if (tmp == NULL || --tmp->valref > 0)
 		return;
-	if ((tmp->flags & MALLOC) != 0) {
-		if (tmp->valref > 1) {
-			tmp->valref--;
-			return;
-		}
-		if ((tmp->flags & STRCUR) != 0)
-			efree(tmp->stptr);
-	}
-#else
-	if ((tmp->flags & (MALLOC|STRCUR)) == (MALLOC|STRCUR))
-		efree(tmp->stptr);
 #endif
 
+	if ((tmp->flags & (MALLOC|STRCUR)) == (MALLOC|STRCUR))
+		efree(tmp->stptr);
+
+	if ((tmp->flags & REGEX) != 0) {
+		refree(tmp->typed_re->re_reg[0]);
+		if (tmp->typed_re->re_reg[1] != NULL)
+			refree(tmp->typed_re->re_reg[1]);
+		unref(tmp->typed_re->re_exp);
+		freenode(tmp->typed_re);
+	}
+
 	mpfr_unset(tmp);
 
+	if (tmp->type == Node_elem_new && tmp->elemnew_vname != NULL)
+		efree(tmp->elemnew_vname);
+	else if ((tmp->type == Node_var || tmp->type == Node_var_new)
+			&& tmp->vname != NULL)
+		efree(tmp->vname);
+
 	free_wstr(tmp);
 	freenode(tmp);
 }
@@ -521,31 +564,32 @@ r_unref(NODE *tmp)
 /*
  * parse_escape:
  *
- * Parse a C escape sequence.  STRING_PTR points to a variable containing a
- * pointer to the string to parse.  That pointer is updated past the
- * characters we use.  The value of the escape sequence is returned.
+ * Parse a C escape sequence.  string_ptr points to a variable containing
+ * a pointer to the string to parse.  result points to a pointer which will
+ * be set to the address of the internal buffer holding the bytes of the
+ * translated escape sequence.
  *
- * A negative value means the sequence \ newline was seen, which is supposed to
- * be equivalent to nothing at all.
+ * Return values:
+ *	ESCAPE_OK,		// nbytes == 1 to MB_CUR_MAX: the length of the translated escape sequence
+ *	ESCAPE_CONV_ERR,	// wcrtomb conversion error
+ *	ESCAPE_TERM_BACKSLASH,	// terminal backslash (to be preserved in cmdline strings)
+ *	ESCAPE_LINE_CONTINUATION	// line continuation  (backslash-newline pair)
  *
- * If \ is followed by a null character, we return a negative value and leave
- * the string pointer pointing at the null character.
- *
- * If \ is followed by 000, we return 0 and leave the string pointer after the
- * zeros.  A value of 0 does not mean end of string.
- *
- * POSIX doesn't allow \x.
+ * POSIX doesn't allow \x or \u.
  */
 
-int
-parse_escape(const char **string_ptr)
+enum escape_results
+parse_escape(const char **string_ptr, const char **result, size_t *nbytes)
 {
+	static char buf[MB_LEN_MAX];
+	enum escape_results retval = ESCAPE_OK;
 	int c = *(*string_ptr)++;
 	int i;
 	int count;
 	int j;
 	const char *start;
 
+	*nbytes = 1;
 	if (do_lint_old) {
 		switch (c) {
 		case 'a':
@@ -559,24 +603,33 @@ parse_escape(const char **string_ptr)
 
 	switch (c) {
 	case 'a':
-		return '\a';
+		buf[0] = '\a';
+		break;
 	case 'b':
-		return '\b';
+		buf[0] = '\b';
+		break;
 	case 'f':
-		return '\f';
+		buf[0] = '\f';
+		break;
 	case 'n':
-		return '\n';
+		buf[0] = '\n';
+		break;
 	case 'r':
-		return '\r';
+		buf[0] = '\r';
+		break;
 	case 't':
-		return '\t';
+		buf[0] = '\t';
+		break;
 	case 'v':
-		return '\v';
+		buf[0] = '\v';
+		break;
 	case '\n':
-		return -2;
+		retval = ESCAPE_LINE_CONTINUATION;
+		break;
 	case 0:
 		(*string_ptr)--;
-		return -1;
+		retval = ESCAPE_TERM_BACKSLASH;
+		break;
 	case '0':
 	case '1':
 	case '2':
@@ -596,7 +649,8 @@ parse_escape(const char **string_ptr)
 				break;
 			}
 		}
-		return i;
+		buf[0] = i;
+		break;
 	case 'x':
 		if (do_lint) {
 			static bool warned = false;
@@ -606,16 +660,19 @@ parse_escape(const char **string_ptr)
 				lintwarn(_("POSIX does not allow `\\x' escapes"));
 			}
 		}
-		if (do_posix)
-			return ('x');
+		if (do_posix) {
+			buf[0] = 'x';
+			break;
+		}
 		if (! isxdigit((unsigned char) (*string_ptr)[0])) {
 			warning(_("no hex digits in `\\x' escape sequence"));
-			return ('x');
+			buf[0] = 'x';
+			break;
 		}
 		start = *string_ptr;
 		for (i = j = 0; j < 2; j++) {
 			/* do outside test to avoid multiple side effects */
-			c = *(*string_ptr)++;
+			c = (unsigned char) *(*string_ptr)++;
 			if (isxdigit(c)) {
 				i *= 16;
 				if (isdigit(c))
@@ -629,12 +686,73 @@ parse_escape(const char **string_ptr)
 				break;
 			}
 		}
-		if (do_lint && j > 2)
-			lintwarn(_("hex escape \\x%.*s of %d characters probably not interpreted the way you expect"), j, start, j);
-		return i;
+		if (do_lint && j == 2 && isxdigit((unsigned char)*(*string_ptr)))
+			lintwarn(_("hex escape \\x%.*s of %d characters probably not interpreted the way you expect"), 3, start, 3);
+		buf[0] = i;
+		break;
+	case 'u':
+	{
+		size_t n;
+#ifndef __MINGW32__
+		mbstate_t mbs;
+#endif
+
+		if (do_lint) {
+			static bool warned = false;
+
+			if (! warned) {
+				warned = true;
+				lintwarn(_("POSIX does not allow `\\u' escapes"));
+			}
+		}
+		if (do_posix) {
+			buf[0] = 'u';
+			break;
+		}
+		if (! isxdigit((unsigned char) (*string_ptr)[0])) {
+			warning(_("no hex digits in `\\u' escape sequence"));
+			buf[0] = 'u';
+			break;
+		}
+		start = *string_ptr;
+		for (i = j = 0; j < 8; j++) {
+			/* do outside test to avoid multiple side effects */
+			c = (unsigned char) *(*string_ptr)++;
+			if (isxdigit(c)) {
+				i *= 16;
+				if (isdigit(c))
+					i += c - '0';
+				else if (isupper(c))
+					i += c - 'A' + 10;
+				else
+					i += c - 'a' + 10;
+			} else {
+				(*string_ptr)--;
+				break;
+			}
+		}
+#ifdef __MINGW32__
+		n = w32_wc_to_lc (i, buf);
+#elif defined (__CYGWIN__)
+		memset(& mbs, 0, sizeof(mbs));
+		n = wcitomb(buf, i, & mbs);
+#else
+		memset(& mbs, 0, sizeof(mbs));
+		n = wcrtomb(buf, i, & mbs);
+#endif	/* !__MINGW32__ */
+		if (n == (size_t) -1) {
+			warning(_("invalid `\\u' escape sequence"));
+			retval = ESCAPE_CONV_ERR;
+			*nbytes = 0;
+		} else {
+			*nbytes = n;
+		}
+		break;
+	}
 	case '\\':
 	case '"':
-		return c;
+		buf[0] = c;
+		break;
 	default:
 	{
 		static bool warned[256];
@@ -648,8 +766,12 @@ parse_escape(const char **string_ptr)
 			warning(_("escape sequence `\\%c' treated as plain `%c'"), uc, uc);
 		}
 	}
-		return c;
+		buf[0] = c;
+		break;
 	}
+
+	*result = buf;
+	return retval;
 }
 
 /* get_numbase --- return the base to use for the number in 's' */
@@ -709,6 +831,20 @@ str2wstr(NODE *n, size_t **ptr)
 
 	assert((n->flags & (STRING|STRCUR)) != 0);
 
+	/*
+	 * For use by do_match, create and fill in an array.
+	 * For each byte `i' in n->stptr (the original string),
+	 * a[i] is equal to `j', where `j' is the corresponding wchar_t
+	 * in the converted wide string.
+	 *
+	 * This is needed even for Nnull_string or Null_field.
+	 *
+	 * Create the array.
+	 */
+	if (ptr != NULL) {
+		ezalloc(*ptr, size_t *, sizeof(size_t) * (n->stlen + 1));
+	}
+
 	/*
 	 * Don't convert global null string or global null field
 	 * variables to a wide string. They are both zero-length anyway.
@@ -739,21 +875,9 @@ str2wstr(NODE *n, size_t **ptr)
 	 * realloc the wide string down in size.
 	 */
 
-	emalloc(n->wstptr, wchar_t *, sizeof(wchar_t) * (n->stlen + 1), "str2wstr");
+	emalloc(n->wstptr, wchar_t *, sizeof(wchar_t) * (n->stlen + 1));
 	wsp = n->wstptr;
 
-	/*
-	 * For use by do_match, create and fill in an array.
-	 * For each byte `i' in n->stptr (the original string),
-	 * a[i] is equal to `j', where `j' is the corresponding wchar_t
-	 * in the converted wide string.
-	 *
-	 * Create the array.
-	 */
-	if (ptr != NULL) {
-		ezalloc(*ptr, size_t *, sizeof(size_t) * n->stlen, "str2wstr");
-	}
-
 	sp = n->stptr;
 	src_count = n->stlen;
 	memset(& mbs, 0, sizeof(mbs));
@@ -781,7 +905,7 @@ str2wstr(NODE *n, size_t **ptr)
 			/* Warn the user something's wrong */
 			if (! warned) {
 				warned = true;
-				warning(_("Invalid multibyte data detected. There may be a mismatch between your data and your locale."));
+				warning(_("Invalid multibyte data detected. There may be a mismatch between your data and your locale"));
 			}
 
 			/*
@@ -823,12 +947,17 @@ str2wstr(NODE *n, size_t **ptr)
 		}
 	}
 
+	/* Needed for zero-length matches at the end of a string */
+	assert(sp - n->stptr == n->stlen);
+	if (ptr != NULL)
+		(*ptr)[sp - n->stptr] = i;
+
 	*wsp = L'\0';
 	n->wstlen = wsp - n->wstptr;
 	n->flags |= WSTRCUR;
 #define ARBITRARY_AMOUNT_TO_GIVE_BACK 100
 	if (n->stlen - n->wstlen > ARBITRARY_AMOUNT_TO_GIVE_BACK)
-		erealloc(n->wstptr, wchar_t *, sizeof(wchar_t) * (n->wstlen + 1), "str2wstr");
+		erealloc(n->wstptr, wchar_t *, sizeof(wchar_t) * (n->wstlen + 1));
 
 	return n;
 }
@@ -855,7 +984,7 @@ wstr2str(NODE *n)
 	memset(& mbs, 0, sizeof(mbs));
 
 	length = n->wstlen;
-	emalloc(newval, char *, (length * gawk_mb_cur_max) + 1, "wstr2str");
+	emalloc(newval, char *, (length * gawk_mb_cur_max) + 1);
 
 	wp = n->wstptr;
 	for (cp = newval; length > 0; length--) {
@@ -963,7 +1092,7 @@ out:	;
 
 /* is_ieee_magic_val --- return true for +inf, -inf, +nan, -nan */
 
-static int
+bool
 is_ieee_magic_val(const char *val)
 {
 	/*
@@ -1018,7 +1147,7 @@ void init_btowc_cache()
 {
 	int i;
 
-	for (i = 0; i < 255; i++) {
+	for (i = 0; i <= 255; i++) {
 		btowc_cache[i] = btowc(i);
 	}
 }
@@ -1028,10 +1157,6 @@ void init_btowc_cache()
 struct block_header nextfree[BLOCK_MAX] = {
 	{ NULL, sizeof(NODE), "node" },
 	{ NULL, sizeof(BUCKET), "bucket" },
-#ifdef HAVE_MPFR
-	{ NULL, sizeof(mpfr_t), "mpfr" },
-	{ NULL, sizeof(mpz_t), "mpz" },
-#endif
 };
 
 #ifdef MEMDEBUG
@@ -1040,7 +1165,7 @@ void *
 r_getblock(int id)
 {
 	void *res;
-	emalloc(res, void *, nextfree[id].size, "getblock");
+	emalloc(res, void *, nextfree[id].size);
 	nextfree[id].active++;
 	if (nextfree[id].highwater < nextfree[id].active)
 		nextfree[id].highwater = nextfree[id].active;
@@ -1070,7 +1195,7 @@ more_blocks(int id)
 	size = nextfree[id].size;
 
 	assert(size >= sizeof(struct block_item));
-	emalloc(freep, struct block_item *, BLOCKCHUNK * size, "more_blocks");
+	emalloc(freep, struct block_item *, BLOCKCHUNK * size);
 	p = (char *) freep;
 	endp = p + BLOCKCHUNK * size;
 
@@ -1088,3 +1213,24 @@ more_blocks(int id)
 }
 
 #endif
+
+/* make_bool_node --- make a boolean-valued node */
+
+extern NODE *
+make_bool_node(bool value)
+{
+	NODE *val;
+	const char *sval;
+	AWKNUM nval;
+
+	sval = (value ? "1" : "0");
+	nval = (value ? 1.0 : 0.0);
+
+	val = make_number(nval);
+	val->stptr = estrdup(sval, strlen(sval));
+	val->stlen = strlen(sval);
+	val->flags |= NUMCUR|STRCUR|BOOLVAL;
+	val->stfmt = STFMT_UNUSED;
+
+	return val;
+}
