@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 1999-2022 the Free Software Foundation, Inc.
+ * Copyright (C) 1999-2023, 2025, the Free Software Foundation, Inc.
  *
  * This file is part of GAWK, the GNU implementation of the
  * AWK Programming Language.
@@ -67,6 +67,8 @@ static long indent_level = 0;
 static const char tabs[] = "\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t";
 static const size_t tabs_len = sizeof(tabs) - 1;
 
+static bool at_start = true;
+
 #define check_indent_level() \
 	if (indent_level + 1 > tabs_len) \
 		/* We're allowed to be snarky, occasionally. */ \
@@ -114,6 +116,17 @@ set_prof_file(const char *file)
 	}
 }
 
+/* close_prof_file --- close the output file for profiling or pretty-printing */
+
+void
+close_prof_file(void)
+{
+	if (prof_fp != NULL
+	    && fileno(prof_fp) != fileno(stdout)
+	    && fileno(prof_fp) != fileno(stderr))
+		(void) fclose(prof_fp);
+}
+
 /* init_profiling_signals --- set up signal handling for gawk --profile */
 
 void
@@ -171,6 +184,7 @@ pp_push(int type, char *s, int flag, INSTRUCTION *comment)
 {
 	NODE *n;
 	getnode(n);
+	memset(n, '\0', sizeof(NODE));
 	n->pp_str = s;
 	n->pp_len = strlen(s);
 	n->flags = flag;
@@ -269,7 +283,11 @@ pprint(INSTRUCTION *startp, INSTRUCTION *endp, int flags)
 					if (! rule_count[rule]++)
 						fprintf(prof_fp, _("\t# %s rule(s)\n\n"), ruletab[rule]);
 					indent(0);
-				}
+				} else if (! at_start)
+					putc('\n', prof_fp);
+				else
+					at_start = false;
+
 				fprintf(prof_fp, "%s {", ruletab[rule]);
 				end_line(pc);
 				skip_comment = true;
@@ -277,6 +295,10 @@ pprint(INSTRUCTION *startp, INSTRUCTION *endp, int flags)
 				if (do_profile && ! rule_count[rule]++)
 					fprintf(prof_fp, _("\t# Rule(s)\n\n"));
 				ip1 = pc->nexti;
+				if (! at_start)
+					putc('\n', prof_fp);
+				else
+					at_start = false;
 				indent(ip1->exec_count);
 				if (ip1 != (pc + 1)->firsti) {		/* non-empty pattern */
 					pprint(ip1->nexti, (pc + 1)->firsti, NO_PPRINT_FLAGS);
@@ -308,7 +330,7 @@ pprint(INSTRUCTION *startp, INSTRUCTION *endp, int flags)
 			indent_out();
 			if (do_profile)
 				indent(0);
-			fprintf(prof_fp, "}\n\n");
+			fprintf(prof_fp, "}\n");
 			pc = (pc + 1)->lasti;
 			break;
 
@@ -432,7 +454,7 @@ cleanup:
 						+ indent_level + 1				// indent
 						+ pc->comment->memory->stlen + 3;		// tab comment
 
-				emalloc(str, char *, len, "pprint");
+				emalloc(str, char *, len);
 				sprintf(str, "%s%s%s%.*s %s", t1->pp_str, op2str(pc->opcode),
 						pc->comment->memory->stptr,
 						(int) (indent_level + 1), tabs, t2->pp_str);
@@ -539,15 +561,28 @@ cleanup:
 			break;
 
 		case Op_store_field:
+		case Op_store_field_exp:
+		{
+			char *assignment, *final;
+
 			t1 = pp_pop(); /* field num */
 			if (is_binary(t1->type))
 				pp_parenthesize(t1);
 			t2 = pp_pop(); /* r.h.s. */
-			fprintf(prof_fp, "$%s%s%s", t1->pp_str, op2str(pc->opcode), t2->pp_str);
+			assignment = pp_group3(t1->pp_str, op2str(pc->opcode), t2->pp_str);
+			final = pp_group3("$", assignment, "");
+			efree(assignment);
 			pp_free(t2);
 			pp_free(t1);
-			if ((flags & IN_FOR_HEADER) == 0)
-				pc = end_line(pc);
+			if (pc->opcode == Op_store_field_exp)
+				pp_push(pc->opcode, final, CAN_FREE, NULL);
+			else {
+				fprintf(prof_fp, "%s", final);
+				efree(final);
+				if ((flags & IN_FOR_HEADER) == 0)
+					pc = end_line(pc);
+			}
+		}
 			break;
 
 		case Op_concat:
@@ -611,6 +646,7 @@ cleanup:
 		case Op_newfile:
 		case Op_get_record:
 		case Op_lint:
+		case Op_lint_plus:
 		case Op_jmp:
 		case Op_jmp_false:
 		case Op_jmp_true:
@@ -669,7 +705,7 @@ cleanup:
 				NODE *n = pp_top();
 
 				if (pc->expr_count == 1
-				    && n->pp_str[0] == '(' 
+				    && n->pp_str[0] == '('
 				    && n->pp_str[n->pp_len - 1] == ')') {
 					n = pp_pop();
 
@@ -699,8 +735,7 @@ cleanup:
 		case Op_push_re:
 			if (pc->memory->type != Node_regex && (pc->memory->flags & REGEX) == 0)
 				break;
-			/* else
-				fall through */
+			/* fall through */
 		case Op_match_rec:
 		{
 			if (pc->memory->type == Node_regex) {
@@ -1140,7 +1175,7 @@ cleanup:
 			len = f->pp_len + t->pp_len + cond->pp_len + 12;
 			if (qm_comment == NULL && colon_comment == NULL) {
 				// easy case
-				emalloc(str, char *, len, "pprint");
+				emalloc(str, char *, len);
 				sprintf(str, "%s ? %s : %s", cond->pp_str, t->pp_str, f->pp_str);
 			} else if (qm_comment != NULL && colon_comment != NULL) {
 				check_indent_level();
@@ -1148,7 +1183,7 @@ cleanup:
 					colon_comment->memory->stlen +
 					2 * (indent_level + 1) + 3 +		// indentation
 					t->pp_len + 6;
-				emalloc(str, char *, len, "pprint");
+				emalloc(str, char *, len);
 				sprintf(str,
 					"%s ? %s"	// cond ? comment
 					"%.*s   %s"	// indent true-part
@@ -1167,7 +1202,7 @@ cleanup:
 				len += qm_comment->memory->stlen +	// comment
 					1 * (indent_level + 1) + 3 +	// indentation
 					t->pp_len + 3;
-				emalloc(str, char *, len, "pprint");
+				emalloc(str, char *, len);
 				sprintf(str,
 					"%s ? %s"	// cond ? comment
 					"%.*s   %s"	// indent true-part
@@ -1183,7 +1218,7 @@ cleanup:
 				len += colon_comment->memory->stlen +		// comment
 					1 * (indent_level + 1) + 3 +		// indentation
 					t->pp_len + 3;
-				emalloc(str, char *, len, "pprint");
+				emalloc(str, char *, len);
 				sprintf(str,
 					"%s ? %s"	// cond ? true-part
 					" : %s"		// : comment
@@ -1325,7 +1360,7 @@ print_lib_list(FILE *prof_fp)
 		}
 	}
 	if (found)	/* we found some */
-		fprintf(prof_fp, "\n");
+		at_start = false;
 }
 
 /* print_include_list --- print a list of all files included */
@@ -1356,7 +1391,7 @@ print_include_list(FILE *prof_fp)
 		}
 	}
 	if (found)	/* we found some */
-		fprintf(prof_fp, "\n");
+		at_start = false;
 }
 
 /* print_comment --- print comment text with proper indentation */
@@ -1368,6 +1403,13 @@ print_comment(INSTRUCTION* pc, long in)
 	size_t count;
 	bool after_newline = false;
 
+	if (pc->memory->comment_type == BLOCK_COMMENT) {
+		if (! at_start && indent_level == 0)
+			putc('\n', prof_fp);
+		else
+			at_start = false;
+	}
+
 	count = pc->memory->stlen;
 	text = pc->memory->stptr;
 
@@ -1604,7 +1646,7 @@ pp_parenthesize(NODE *sp)
 	if (p[0] == '(')	// already parenthesized
 		return;
 
-	emalloc(p, char *, len + 3, "pp_parenthesize");
+	emalloc(p, char *, len + 3);
 	*p = '(';
 	memcpy(p + 1, sp->pp_str, len);
 	p[len + 1] = ')';
@@ -1677,7 +1719,7 @@ pp_string_or_typed_regex(const char *in_str, size_t len, int delim, bool typed_r
 /* make space for something l big in the buffer */
 #define chksize(l)  if ((l) > ofre) { \
 		long olen = obufout - obuf; \
-		erealloc(obuf, char *, osiz * 2, "pp_string"); \
+		erealloc(obuf, char *, osiz * 2); \
 		obufout = obuf + olen; \
 		ofre += osiz; \
 		osiz *= 2; \
@@ -1685,7 +1727,7 @@ pp_string_or_typed_regex(const char *in_str, size_t len, int delim, bool typed_r
 
 	/* initial size; 3 for delim + terminating null, 1 for @ */
 	osiz = len + 3 + 1 + (typed_regex == true);
-	emalloc(obuf, char *, osiz, "pp_string");
+	emalloc(obuf, char *, osiz);
 	obufout = obuf;
 	ofre = osiz - 1;
 
@@ -1738,7 +1780,7 @@ pp_number(NODE *n)
 	char *str;
 
 	assert((n->flags & NUMCONSTSTR) != 0);
-	emalloc(str, char *, n->stlen + 1, "pp_number");
+	emalloc(str, char *, n->stlen + 1);
 	strcpy(str, n->stptr);
 	return str;
 }
@@ -1770,10 +1812,10 @@ pp_list(int nargs, const char *paren, const char *delim)
 
 	if (pp_args == NULL) {
 		npp_args = nargs;
-		emalloc(pp_args, NODE **, (nargs + 2) * sizeof(NODE *), "pp_list");
+		emalloc(pp_args, NODE **, (nargs + 2) * sizeof(NODE *));
 	} else if (nargs > npp_args) {
 		npp_args = nargs;
-		erealloc(pp_args, NODE **, (nargs + 2) * sizeof(NODE *), "pp_list");
+		erealloc(pp_args, NODE **, (nargs + 2) * sizeof(NODE *));
 	}
 
 	delimlen = strlen(delim);
@@ -1796,7 +1838,7 @@ pp_list(int nargs, const char *paren, const char *delim)
 	}
 	comment = NULL;
 
-	emalloc(str, char *, len + 1, "pp_list");
+	emalloc(str, char *, len + 1);
 	s = str;
 	if (paren != NULL)
 		*s++ = paren[0];
@@ -1853,10 +1895,10 @@ pp_concat(int nargs)
 
 	if (pp_args == NULL) {
 		npp_args = nargs;
-		emalloc(pp_args, NODE **, (nargs + 2) * sizeof(NODE *), "pp_concat");
+		emalloc(pp_args, NODE **, (nargs + 2) * sizeof(NODE *));
 	} else if (nargs > npp_args) {
 		npp_args = nargs;
-		erealloc(pp_args, NODE **, (nargs + 2) * sizeof(NODE *), "pp_concat");
+		erealloc(pp_args, NODE **, (nargs + 2) * sizeof(NODE *));
 	}
 
 	/*
@@ -1870,7 +1912,7 @@ pp_concat(int nargs)
 		len += r->pp_len + delimlen + 2;
 	}
 
-	emalloc(str, char *, len + 1, "pp_concat");
+	emalloc(str, char *, len + 1);
 	s = str;
 
 	/* now copy in */
@@ -1944,7 +1986,7 @@ pp_group3(const char *s1, const char *s2, const char *s3)
 	len2 = strlen(s2);
 	len3 = strlen(s3);
 	l = len1 + len2 + len3 + 1;
-	emalloc(str, char *, l, "pp_group3");
+	emalloc(str, char *, l);
 	s = str;
 	if (len1 > 0) {
 		memcpy(s, s1, len1);
@@ -1990,7 +2032,7 @@ pp_func(INSTRUCTION *pc, void *data ATTRIBUTE_UNUSED)
 		print_comment(pc->comment, -1);	/* -1 ==> don't indent */
 
 	indent(pc->nexti->exec_count);
-	
+
 	bool malloced = false;
 	char *name = adjust_namespace(func->vname, & malloced);
 	fprintf(prof_fp, "%s %s(", op2str(Op_K_function), name);
@@ -2018,6 +2060,7 @@ pp_func(INSTRUCTION *pc, void *data ATTRIBUTE_UNUSED)
 	if (do_profile)
 		indent(0);
 	fprintf(prof_fp, "}\n");
+	at_start = false;
 	return 0;
 }
 
@@ -2058,8 +2101,8 @@ pp_namespace(const char *name, INSTRUCTION *comment)
 	// info saved in Op_namespace instructions.
 	current_namespace = name;
 
-	// force newline, could be after a comment
-	fprintf(prof_fp, "\n");
+	if (! at_start)
+		fprintf(prof_fp, "\n");
 
 	if (do_profile)
 		indent(SPACEOVER);
@@ -2069,9 +2112,11 @@ pp_namespace(const char *name, INSTRUCTION *comment)
 	if (comment != NULL) {
 		putc('\t', prof_fp);
 		print_comment(comment, 0);
-		putc('\n', prof_fp);
+		// no newline here, print_comment puts one out
 	} else
-		fprintf(prof_fp, "\n\n");
+		fprintf(prof_fp, "\n");
+
+	at_start = false;
 }
 
 /* pp_namespace_list --- print the list, back to front, using recursion */
@@ -2101,7 +2146,7 @@ adjust_namespace(char *name, bool *malloced)
 		char *buf;
 		size_t len = 5 + strlen(name) + 1;
 
-		emalloc(buf, char *, len, "adjust_namespace");
+		emalloc(buf, char *, len);
 		sprintf(buf, "awk::%s", name);
 		*malloced = true;
 
