@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 1991-2019, 2021-2024
+ * Copyright (C) 1991-2019, 2021-2025
  * the Free Software Foundation, Inc.
  *
  * This file is part of GAWK, the GNU implementation of the
@@ -82,10 +82,10 @@ make_regexp(const char *s, size_t len, bool ignorecase, bool dfa, bool canfatal)
 	 * from that.
 	 */
 	if (buf == NULL) {
-		emalloc(buf, char *, len + 1, "make_regexp");
+		emalloc(buf, char *, len + 1);
 		buflen = len;
 	} else if (len > buflen) {
-		erealloc(buf, char *, len + 1, "make_regexp");
+		erealloc(buf, char *, len + 1);
 		buflen = len;
 	}
 	dest = buf;
@@ -261,9 +261,9 @@ make_regexp(const char *s, size_t len, bool ignorecase, bool dfa, bool canfatal)
 	*dest = '\0';
 	len = dest - buf;
 
-	ezalloc(rp, Regexp *, sizeof(*rp), "make_regexp");
+	ezalloc(rp, Regexp *, sizeof(*rp));
 	rp->pat.allocated = 0;	/* regex will allocate the buffer */
-	emalloc(rp->pat.fastmap, char *, 256, "make_regexp");
+	emalloc(rp->pat.fastmap, char *, 256);
 
 	/*
 	 * Lo these many years ago, had I known what a P.I.T.A. IGNORECASE
@@ -333,7 +333,7 @@ make_regexp(const char *s, size_t len, bool ignorecase, bool dfa, bool canfatal)
 	}
 
 	for (i = len - 1; i >= 0; i--) {
-		if (strchr("*+|?{}", buf[i]) != NULL) {
+		if (strchr("\\*+|?{}", buf[i]) != NULL) {
 			rp->maybe_long = true;
 			break;
 		}
@@ -598,6 +598,7 @@ reflags2str(int flagval)
 		{ RE_UNMATCHED_RIGHT_PAREN_ORD, "RE_UNMATCHED_RIGHT_PAREN_ORD" },
 		{ RE_NO_POSIX_BACKTRACKING, "RE_NO_POSIX_BACKTRACKING" },
 		{ RE_NO_GNU_OPS, "RE_NO_GNU_OPS" },
+		{ RE_DEBUG, "RE_DEBUG" },	// not actually used in the code anymore, :-(
 		{ RE_INVALID_INTERVAL_ORD, "RE_INVALID_INTERVAL_ORD" },
 		{ RE_ICASE, "RE_ICASE" },
 		{ RE_CARET_ANCHORS_HERE, "RE_CARET_ANCHORS_HERE" },
@@ -674,23 +675,26 @@ again:
 	if (sp == NULL)
 		goto done;
 
-	for (count++, sp++; *sp != '\0'; sp++) {
+	sp++;
+	count = 1;
+	/*
+	 * Skip over the following:
+	 * [^]...]
+	 * [\]...]
+	 * []...]
+	 */
+	if (*sp == '^')
+		sp++;
+	if (*sp == '\\')
+		sp += 2;
+	else if (*sp == ']')
+		sp++;
+
+	for (; sp < end && *sp != '\0'; sp++) {
 		if (*sp == '[')
 			count++;
-		/*
-		 * ] as first char after open [ is skipped
-		 * \] is skipped
-		 * [^]] is skipped
-		 */
-		if (*sp == ']' && sp > sp2) {
-			if (sp[-1] != '[' && sp[-1] != '\\')
-				count--;
-			else if ((sp - sp2) >= 2
-				&& sp[-1] == '^' && sp[-2] == '[')
-				;
-			else
-				count--;
-		}
+		else if (*sp == ']')
+			count--;
 
 		if (count == 0) {
 			sp++;	/* skip past ']' */
